#include <stdio.h>
#include <stdlib.h>
#include "types.h"
#include "defaults.h"
#include "warning.h"
#include "mystring.h"
#include "X.h"
#include "pixmap.h"
#include "window.h"
#include "widget.h"
#include "edit.h"
#include "popup.h"

//: win

Window win = None;
Window textwin = None;

//: real_win

static Window real_win;
static Window okrajwin;

//: vyskasirka

static int real_win_vyska, real_win_sirka, win_x, win_y;

//: nastavicon

static void nastavicon()
{
  int orat;
  XWMHints wmhints;
  XClassHint classhint;

  orat = ratio;
  ratio = 1;
  wmhints.icon_pixmap = nactiXPM("icon", NULL, &wmhints.icon_mask, NULL);
  ratio = orat;

  wmhints.flags = IconPixmapHint|IconMaskHint;
  wmhints.input = 1;

  classhint.res_name = gargv[0];
  classhint.res_class = ICON_NAME;

  XSetClassHint(display, real_win, &classhint);
  XSetWMHints(display, real_win, &wmhints);
}

//: umistiwin

static void umistiwin ()  // umisti okno doprostred obrazovky (ve fullscreenu).
{
  // Spocitam win_x, win_y

  win_x = (display_sirka-pozadi.PIX_SIR*ratio)/2;
  win_y = (display_vyska-OKRAJ-text_vyska-pozadi.PIX_VYS*ratio)/2;

  // Posunu win na souradnice win_x, win_y

  XMoveWindow(display, win, win_x, win_y);
  if(!okrajtext) okraj_to_win();  // Pokud je kolem win okraj, posunu ho take.
}

//: spoctifr

static int spoctifr ()
{
  int pratio;  // pokusne ratio

  // Spocitam maximalni ratio, se kterym se win vejde na obrazovku:

  pratio = (display_sirka-2*OKRAJ)/pozadi.PIX_SIR;
  if(pratio > (display_vyska-text_vyska-3*OKRAJ)/pozadi.PIX_VYS)
    pratio = (display_vyska-text_vyska-3*OKRAJ)/pozadi.PIX_VYS;
  if(pratio < MINRATIO) pratio = MINRATIO;
  if(pratio > MAXRATIO) pratio = MAXRATIO;

  return pratio;
}  

//: fixujokno

static void fixujokno ()
{
  XSizeHints size_hints;

  // Nastavim maximalni i minimalni velikost okna jako aktualni velikost:

  size_hints.flags = PSize | PMinSize | PMaxSize;
  size_hints.width=size_hints.min_width=size_hints.max_width=real_win_sirka;
  size_hints.height=size_hints.min_height=size_hints.max_height=real_win_vyska;

  XSetWMNormalHints(display, real_win, &size_hints);
} 

//: odfixujokno

static void odfixujokno ()
{
  XSizeHints size_hints;

  size_hints.flags = 0;

  XSetWMNormalHints(display, real_win, &size_hints);
} 

//: real_win_to_win

static void real_win_to_win ()  // Zmeni velikost okna podle velikosti win.
{
  // Spocitam real_win_sirka, real_win_vyska:

  real_win_sirka = win_sirka+2*OKRAJ;
  real_win_vyska = win_vyska+text_vyska+3*OKRAJ;
 
  odfixujokno();
  XResizeWindow(display, real_win, real_win_sirka, real_win_vyska); // Zmenim velikost okna
  fixujokno();  // Zaridim, aby nova velikost okna nesla menit.

  // Nastavim sirku textu:

  text_sirka = win_sirka;
  XMoveResizeWindow(display, textwin, OKRAJ, win_vyska+2*OKRAJ,
		    text_sirka, text_vyska);
  vyrestext();

  // Upravim okraj:

  if(!okrajtext) okraj_to_win();
}

/*
Funkce velikokno zmeni velikost win. Pouziva se pri zmene pozadi. Parametr
fixratio je od toho, aby pri chtene zmene velikosti ve fullscreenu se automaticky
nenastavila zpatky puvodni velikost. Pokud je tedy parametr fixratio nastaveny na true,
nebudu hybat se zvetsenim.
*/

//: velikokno

static void velikokno (int fixratio)
{
  int pratio;

  if(fullscreened && !fixratio){
    pratio = spoctifr();
    if(pratio != ratio){
      rescale(pratio);
      return;
    }
  }

  win_sirka = pozadi.PIX_SIR*ratio;
  win_vyska = pozadi.PIX_VYS*ratio;
  XResizeWindow(display, win, win_sirka, win_vyska);

  if(fullscreened) umistiwin();
  else real_win_to_win();
}

//: send_fs_event

static void send_fs_event()
{
  Atom wmState = XInternAtom(display, "_NET_WM_STATE", 0);
  Atom fullScreen = XInternAtom(display,
				"_NET_WM_STATE_FULLSCREEN", 0);

  // Vytvorim fullscreenovaci udalost:

  XEvent xev;
  xev.xclient.type=ClientMessage;
  xev.xclient.serial = 0;
  xev.xclient.send_event=True;
  xev.xclient.window=real_win;
  xev.xclient.message_type=wmState;
  xev.xclient.format=32; 
  xev.xclient.data.l[0] = (fullscreened ? 1 : 0);
  xev.xclient.data.l[1] = fullScreen;
  xev.xclient.data.l[2] = 0;

  // Poslu fullscreenovaci udalost:

  XSendEvent(display, root, False,
	     SubstructureRedirectMask | SubstructureNotifyMask, &xev);
}  

//: fullscreen

void fullscreen ()
{
  int pratio;

  fullscreened = !fullscreened;

  odfixujokno();

  // zmenim velikost okna

  if(fullscreened){
    real_win_sirka = display_sirka;
    real_win_vyska = display_vyska;
    XResizeWindow(display, real_win, real_win_sirka, real_win_vyska);
  }

  send_fs_event();  // Poslu fullscreenovaci udalost

  if(fullscreened){
    // Dam textwin dospod obrazovky.
    text_sirka = real_win_sirka-2*OKRAJ;
    XMoveResizeWindow(display, textwin,
		      OKRAJ, real_win_vyska-OKRAJ-text_vyska,
		      text_sirka, text_vyska);
    vyrestext();

    unfullratio = ratio; // Zazalohuji ratio.

    pratio = spoctifr(); // Spocitam maximalni ratio, se kterym se win vejde na obrazovku

    if(pratio == ratio) umistiwin(); /* Pokud vyjde stejne jako puvodni zvetseni,
				        jen dam win doprostred */
    else rescale(pratio);  // jinak prescaluji hru
  }
  else{
    // Posunu win zpatky do leveho horniho rohu:

    win_x = win_y = OKRAJ;
    XMoveWindow(display, win, win_x, win_y);

    // Vratim puvodni zvetseni:

    if(unfullratio == ratio) real_win_to_win();
    else rescale(unfullratio);
  }
}

//: okraj_to

void okraj_to_win ()
{
  XMoveResizeWindow(display, okrajwin,
		    win_x-OKRAJ, win_y-OKRAJ, win_sirka+2*OKRAJ, win_vyska+2*OKRAJ);
  okrajtext = 0;
  mengraf = 1;
}

void okraj_to_text ()
{
  XMoveResizeWindow(display, okrajwin,
		    0, real_win_vyska-2*OKRAJ-text_vyska, real_win_sirka, text_vyska+2*OKRAJ);
  okrajtext = 1;
}

//: vytvorokno

void vytvorokno (char *pnazev)
/****************************/
{
  char *fullname;

  loadfont();  // Nactu font.
  loadpopupfont();  // Nactu font pro vyskakovaci okna.

  // Nastavim udaje o oknech:

  fullname = joinstrings(BGDIR, pnazev, NULL);
  if(fullscreened){
    if(!unfullratio) unfullratio = ratio;
    ratio = 0;
    pozadi.barvy = nactiXPM(fullname, &pozadi.ibarvy, NULL, NULL);
    ratio = spoctifr();                   // spocitam ratio pro fullscreen
    if(!unfullratio) unfullratio = ratio;
    rescalePixmap(pozadi.ibarvy, &pozadi.barvy);

    win_x = (display_sirka-pozadi.PIX_SIR*ratio)/2;
    win_y = (display_vyska-OKRAJ-text_vyska-pozadi.PIX_VYS*ratio)/2;
    win_sirka = pozadi.PIX_SIR*ratio;
    win_vyska = pozadi.PIX_VYS*ratio;
    real_win_sirka = display_sirka;
    real_win_vyska = display_vyska;
    text_sirka = display_sirka-2*OKRAJ;
  }
  else{
    pozadi.barvy = nactiXPM(fullname, &pozadi.ibarvy, NULL, NULL);
    win_x = win_y = OKRAJ;
    win_sirka = pozadi.PIX_SIR*ratio;
    win_vyska = pozadi.PIX_VYS*ratio;
    real_win_sirka = win_sirka+2*OKRAJ;
    real_win_vyska = win_vyska+text_vyska+3*OKRAJ;
    text_sirka = win_sirka;
  }
  free(fullname);

  // Vytvorim okna:

  real_win = XCreateSimpleWindow(display, root,
				 0, 0,
				 real_win_sirka, real_win_vyska,
				 0, BlackPixel(display,screen), BlackPixel(display,screen));

  if(okrajtext){
    okrajwin = XCreateSimpleWindow(display, real_win,
				   0, real_win_vyska - (text_vyska+2*OKRAJ),
				   real_win_sirka, text_vyska+2*OKRAJ,
				   0, WhitePixel(display,screen), WhitePixel(display,screen));
  }
  else{
    okrajwin = XCreateSimpleWindow(display, real_win,
				   win_x-OKRAJ, win_y-OKRAJ,
				   win_sirka+2*OKRAJ, win_vyska+2*OKRAJ,
				   0, WhitePixel(display,screen), WhitePixel(display,screen));
  }

  win = XCreateSimpleWindow(display, real_win,
			    win_x, win_y, win_sirka, win_vyska, 0,
			    BlackPixel(display,screen), BlackPixel(display,screen));


  textwin = XCreateSimpleWindow(display, real_win,
				OKRAJ, real_win_vyska-text_vyska-OKRAJ,
				text_sirka, text_vyska, 0,
				WhitePixel(display,screen), BlackPixel(display,screen));

  XSetWindowBackgroundPixmap(display, win, pozadi.barvy); // Nastavim pozadi.

  // Nastavim prijimani udalosti

  XSelectInput(display, real_win, KeyPressMask | ButtonPressMask);
  XSelectInput(display, textwin, ExposureMask);

  // Vytvorim text_gc:

  text_gc = XCreateGC(display, textwin, 0, NULL);
  XSetFont(display, text_gc, font_info->fid);
  XSetForeground(display, text_gc, WhitePixel(display,screen));  

  // Nastavim udaje o oknu

  XSetStandardProperties(display, real_win, WINDOW_NAME, ICON_NAME, None,
			   gargv, gargc, NULL);
  if(fullscreened){
    send_fs_event();
  }
  else fixujokno();

  nastavicon();

  // Placnu okno na obrazovku:

  XMapWindow(display, real_win);
  XMapWindow(display, okrajwin);
  XMapWindow(display, textwin);
  XMapWindow(display, win);
  XFlush(display);
}

//: zmenokno

void zmenokno (char *pnazev) // Zmeni pozadi.
{
  int sirka, vyska;
  char *fullname;

  // Ulozim puvodni sirku a vysku:

  sirka = pozadi.PIX_SIR;
  vyska = pozadi.PIX_VYS;

  // Smazu puvodni obrazky:

  XFreePixmap(display, pozadi.barvy);
  XDestroyImage(pozadi.ibarvy);

  fullname = joinstrings(BGDIR, pnazev, NULL);
  pozadi.barvy = nactiXPM(fullname, &pozadi.ibarvy, NULL, NULL); // Nactu novou pixmapu.
  free(fullname);

  XSetWindowBackgroundPixmap(display, win, pozadi.barvy);      // Nastavim novou pixmapu.

  // Vyresim ruznou velikost noveho a stareho pozadi:

  if(sirka != pozadi.PIX_SIR || vyska != pozadi.PIX_VYS) velikokno(0);

  // Nakreslim novou pixmapu:

  XCopyArea(display, pozadi.barvy, win, ggc,
	    0, 0,
	    sirka*ratio, vyska*ratio,
	    0, 0);

  mengraf = 1;
}

//: rescaleokno

void rescaleokno () // Prescaluje pozadi
{
  rescalePixmap(pozadi.ibarvy, &pozadi.barvy);            // Prescaluje pixmapu pozadi
  XSetWindowBackgroundPixmap(display, win, pozadi.barvy); // Nastavi nove pozadi

  velikokno(1); // Vyresi zmenu velikosti pozadi.

  XCopyArea(display, pozadi.barvy, win, ggc,  // Prekresli pozadi.
	    0, 0,
	    pozadi.PIX_SIR*ratio, pozadi.PIX_VYS*ratio,
	    0, 0);

  mengraf = 1;  // Zmenila se grafika, vice viz X.c/kreslianim();
}
